﻿using System;
using System.Windows.Controls;

namespace Qnomad
{

	public abstract class DataValidationException : Exception
	{
		protected DataValidationException( string message ) : base( message ) { }
		protected DataValidationException( string message, Exception inner ) : base( message, inner ) { }

		public void InvalidateText( Control textControl )
		{
			var be = textControl.GetBindingExpression( TextBox.TextProperty );
			var er = new ValidationError( new ExceptionValidationRule(), be, Message, this );
			Validation.MarkInvalid( be, er );
		}
	}

	public class NotUniqueException : DataValidationException
	{
		public NotUniqueException( string message, Exception inner )
			: base( message ?? "An item with the same value already exists", inner ) { }
		public NotUniqueException( string message ) : this( message, null ) { }
		public NotUniqueException( Exception inner ) : this( null, inner ) { }
		public NotUniqueException() : this( null, null ) { }
	}

	public class StringTruncatedException : DataValidationException
	{
		public StringTruncatedException( string message, Exception inner )
			: base( message ?? "The text is too long", inner ) { }
		public StringTruncatedException( string message ) : this( message, null ) { }
		public StringTruncatedException( Exception inner ) : this( null, inner ) { }
		public StringTruncatedException() : this( null, null ) { }
	}
}
