﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Windows;
using System.Windows.Threading;
using Qnomad.PresentationModel;

namespace Qnomad.Presentation
{
	public class MessageBoxHelper
	{
		public static readonly DependencyProperty MessageProperty = DependencyProperty.RegisterAttached(
			"Message", typeof( DialogPM ), typeof( MessageBoxHelper ), new UIPropertyMetadata( Message_Changed )
		);

		public static DialogPM GetMessage( FrameworkElement elem )
		{
			return (DialogPM)elem.GetValue( MessageProperty );
		}

		public static void SetMessage( FrameworkElement elem, DialogPM value )
		{
			elem.SetValue( MessageProperty, value );
		}

		static void Message_Changed( DependencyObject d, DependencyPropertyChangedEventArgs e )
		{
			if ( e.NewValue != null ) {
				var newValue = (DialogPM)e.NewValue;
				var buttons = GetButtons( newValue.Buttons.Select( pm => pm.DisplayName ).ToList() );

				// Invoke at Render priority so that Message binding has time to update.
				d.Dispatcher.BeginInvoke( DispatcherPriority.Render, (ThreadStart)(() =>
					ExecuteCommand( MessageBox.Show( newValue.Text, newValue.Caption, buttons ), newValue.Buttons )
				) );
			}
		}

		static MessageBoxButton GetButtons( IList<string> realButtons )
		{
			var forComparision = new List<string> { DialogPM.Okay };
			if ( realButtons.EqualsIgnoreOrder( forComparision ) )
				return MessageBoxButton.OK;

			forComparision.Add( DialogPM.Cancel );
			if ( realButtons.EqualsIgnoreOrder( forComparision ) )
				return MessageBoxButton.OKCancel;

			forComparision.Clear();
			forComparision.Add( DialogPM.Yes );
			forComparision.Add( DialogPM.No );
			if ( realButtons.EqualsIgnoreOrder( forComparision ) )
				return MessageBoxButton.YesNo;

			forComparision.Add( DialogPM.Cancel );
			if ( realButtons.EqualsIgnoreOrder( forComparision ) )
				return MessageBoxButton.YesNoCancel;

			throw new InvalidOperationException( "Invalid button combination specified" );
		}

		static void ExecuteCommand( MessageBoxResult result, IList<CommandPM> buttons )
		{
			if ( result == MessageBoxResult.OK )
				buttons.Single( pm => pm.DisplayName == DialogPM.Okay ).Execute( null );
			else if ( result == MessageBoxResult.Cancel )
				buttons.Single( pm => pm.DisplayName == DialogPM.Cancel ).Execute( null );
			else if ( result == MessageBoxResult.Yes )
				buttons.Single( pm => pm.DisplayName == DialogPM.Yes ).Execute( null );
			else if ( result == MessageBoxResult.No )
				buttons.Single( pm => pm.DisplayName == DialogPM.No ).Execute( null );
			else
				throw new InvalidOperationException( "MessageBoxResult is invalid" );
		}
	}
}
