using System;
using Qnomad.PresentationModel;
using Qnomad.Outlining;

namespace Qnomad.Demos.PresentationModel.GUI.Model {

	public sealed class SampleTreeNodePM : HierarchicalPM<SampleTreeNode, SampleTreeNodePM> {

		#region Object Creation

		/// <summary>Creates a root-level SampleTreeNodeView.</summary>
		public SampleTreeNodePM() : this( SampleTreeNode.Root ) {
			foreach ( var descendant in DomainItem.Descendants() ) {
				var descendantPM = CreateInstance( descendant );
				Func<SampleTreeNodePM, SampleTreeNode, bool> isMapped = ( pm, dm ) => pm.DomainItem == dm;
				var parentPM = descendant.ParentNode.GetParallelNode( this, isMapped );
				descendantPM.PreparePosition( parentPM, descendant.PositionInList() ).Apply(); //iterate using its index
			}
		}

		/// <summary>
		/// Creates a SampleTreeNodeView that does not (yet) have any sub-items.
		/// </summary>
		/// <param name="domainItem">The SampleTreeNode to be represented by this node</param>
		private SampleTreeNodePM( SampleTreeNode domainItem ) :
		  base( domainItem, domainItem.SubNodes, true, true, true ) { }

		/// <inheritdoc/>
		protected override SampleTreeNodePM CreateInstance( SampleTreeNode domainItem ) {
			return new SampleTreeNodePM( domainItem );
		}

		#endregion

		/// <inheritdoc/>
		protected override HierarchicalPlacementArgs<SampleTreeNodePM> DesiredPosition( SampleTreeNodePM parent ) {
			//subItems.Insert( atPosition, itemToAdd );
			return new HierarchicalPlacementArgs<SampleTreeNodePM>( OutliningCommands.InsertChild, parent, false );
		}
	}
}
