using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;

using Qnomad.Demos.PresentationModel.GUI.Model;
using Qnomad.Outlining;

namespace Qnomad.Demos.PresentationModel.GUI {

	/// <summary>
	/// Interaction logic for Demo2.xaml
	/// </summary>
	public partial class Demo2 : UserControl {
		private SampleTreeNodePM rootWrapper;
		
		public Demo2() {
			InitializeComponent();

			//Build sample presentation-model objects:
			rootWrapper = new SampleTreeNodePM();
			sampleTreeView.ItemsSource = rootWrapper.SubItems; //SampleTreeNode.Root.SubNodes;
		}

		private void addButton_Click( object sender, RoutedEventArgs args ) {
			var target = (SampleTreeNodePM)sampleTreeView.SelectedItem; //get selected item
			var insertArgs = new HierarchicalPlacementArgs<SampleTreeNode>( OutliningCommands.InsertSiblingAfter, target.DomainItem, false );
			insertArgs.ItemToMove = new SampleTreeNode( editItemBox.Text ); //create new item

			//Insert new item into domain-model tree structure:
			insertArgs.Apply();

			//Select new item in UI:
			rootWrapper.FindDescendant( insertArgs.ItemToMove ).IsSelected = true;
			sampleTreeView.Focus();
		}

		private void updateButton_Click( object sender, RoutedEventArgs args ) {
			BindingExpression be = editItemBox.GetBindingExpression( TextBox.TextProperty );
			be.UpdateSource(); //update sampleTreeView.SelectedItem.DomainItem.Name with editItemBox.Text's value
			sampleTreeView.Focus();
		}

		private void demoteButton_Click( object sender, RoutedEventArgs args ) {
			var pmToMove = (SampleTreeNodePM)sampleTreeView.SelectedItem; //get selected item
			var moveArgs = new HierarchicalPlacementArgs<SampleTreeNode>( OutliningCommands.NudgeDemote, pmToMove.DomainItem, false );

			//Move node within domain-model tree structure:
			moveArgs.Apply();

			pmToMove.FocusTreeViewItem( sampleTreeView );
		}

		private void deleteButton_Click( object sender, RoutedEventArgs args ) {
			SampleTreeNodePM selected = (SampleTreeNodePM)sampleTreeView.SelectedItem; //get selected item
			selected.DomainItem.ParentNode = null; //remove subtree
			sampleTreeView.Focus();
		}
	}
}
