/*  
 Copyright (C) 2010 - 2011  Quantum Nomad LLC (http://www.qnomad.com/)

 This program is licensed under version 3 of the GNU General Public License as published
 by the Free Software Foundation.  You should have received a copy of the license along
 with the source code.  If not, an online copy of the license can be found at
 http://www.qnomad.com/wpf/GPL.aspx.
*/

using System;
using System.ComponentModel;

namespace Qnomad.PresentationModel
{
	/// <summary>Presentation-tailored representation of a domain-model type.</summary>
	/// <typeparam name="DMType">The domain-model type to be wrapped</typeparam>
	public abstract class ItemPM<DMType> : ChangeNotifier
		where DMType : class
	{
		#region Constructor

		public DMType DomainItem { get; private set; }

		protected ItemPM( DMType domainItem )
		{
			DomainItem = domainItem;
		}

		#endregion Constructor

		#region 'IsSelected' Property

		private bool isSelected;

		/// <summary>
		/// Is the TreeViewItem associated with this object selected?
		/// </summary>
		/// <seealso href="http://www.codeproject.com/KB/WPF/TreeViewWithViewModel.aspx"/>
		public bool IsSelected
		{
			get { return isSelected; }
			set
			{
				if ( SetProperty( ref isSelected, value, "IsSelected" ) )
					OnIsSelectedChanged( value );
			}
		}

		protected virtual void OnIsSelectedChanged( bool newValue ) { }

		#endregion IsSelected

		public override string ToString()
		{
			return GetType().Name + ": DomainItem={" + DomainItem + "}";
		}
	}
}
